
# q-expansion of newform 4080.2.a.l, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4080
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [1], [-2], [0], [-4], [-1], [4], [0], [6], [4], [2], [0], [-2], [0], [-6], [-6], [-10], [-2], [6], [-16], [-8], [0], [6], [-4], [12], [16], [-12], [2], [-6], [-8], [-12], [-18], [4], [12], [-8], [20], [16], [-12], [-6], [-18], [-22], [-12], [-4], [-6], [-20], [4], [-8], [12], [14], [6], [-24], [26], [-30], [-30], [0], [6], [-8], [-10], [6], [4], [6], [-2], [-6], [20], [-30], [-32], [32], [-12], [26], [-30], [12], [-14], [32], [4], [-24], [-12], [2], [-12], [-10], [36], [-10], [18], [-10], [16], [-24], [-12], [-10], [0], [-8], [36], [30], [-2], [-18], [28], [24], [12], [-36], [34], [38], [-20], [-6], [-24], [-30], [-20], [-34], [-24], [18], [-12], [-34], [22], [-40], [30], [28], [-8], [-12], [40], [-48], [30], [30], [26], [-4], [-6], [12], [4], [24], [38], [6], [16], [8], [40], [-36], [-32], [20], [18], [14], [-6], [-32], [30], [24], [-20], [18], [-14], [12], [50], [-54], [38], [-18], [40], [48], [14], [-12], [34], [24], [16], [-18], [16], [-48], [-10], [42], [-36], [30], [4], [30], [-30], [60], [52], [-46], [2], [-42], [-30], [-34], [18], [-34], [40], [0], [28], [-30], [-20], [-34], [-8], [-12], [26], [-42], [0], [-60], [-22], [10], [38], [54], [-4], [36], [-32], [24], [-12], [42], [-34], [26], [-30], [24], [-6], [28], [-16], [38], [-30], [6], [-8], [-36], [36], [-56], [8], [24], [-62], [-48], [18], [-58], [16], [6], [0], [-6], [62], [-68], [-30], [46], [-24], [-10], [66], [30], [-32], [-24], [-52], [-20], [16], [-6], [-26], [48], [14], [-54], [-60], [-12], [60], [40], [76], [2], [6], [18], [22], [72], [40], [-24], [-4], [0], [-24], [14], [-54], [-30], [74], [76], [-18], [62], [46], [-36], [-10], [-10], [66], [28], [12], [-54], [-56], [-30], [14], [-38], [38], [-56], [-58], [52], [0], [-46], [38], [-18], [-24], [40], [-72], [14], [40], [-36], [20], [30], [16], [42], [-18], [12], [-42], [-60], [-46], [-66], [-80], [54], [-48], [58], [-22], [-30], [40], [60], [4], [-16], [-48], [50], [72], [-76], [-12], [-42], [24], [58], [-48], [-22], [-6], [54], [-28], [-6], [-20], [32], [84], [-56], [78], [2], [52], [-80], [-24], [6], [-70], [54], [28], [12], [-44], [-12], [-34], [30], [-70], [52], [8], [42], [-6], [-8], [-78], [-24], [-70], [-38], [-30], [-54], [40], [68], [-60], [-86], [-46], [66], [-12], [-72], [-6], [-96], [26], [48], [0], [0], [70], [62], [30], [-56], [-34], [-42], [28], [96], [36], [-32], [14], [24], [42], [2], [-6], [62], [30], [-6], [-38], [-66], [-20], [84], [16], [-4], [-20], [0], [-82], [-6], [-66], [-62], [-48], [-40], [40], [66], [-20], [-60], [-34], [54], [-32], [18], [96], [-104], [-52], [42], [22], [84], [44], [78], [96], [-20], [98], [-78], [-6], [70], [-6], [-48], [96], [-82], [96], [-102], [-76], [30], [-12], [-72], [-104], [-42]]
